@@ -15,7 +15,9 @@
 
 import logging
 
-from os_client_config.config import OpenStackConfig
+from openstack.config import exceptions as sdk_exceptions
+from openstack.config import loader as config
+from oslo_utils import strutils
 
 
 LOG = logging.getLogger(__name__)
@@ -23,7 +25,7 @@ LOG = logging.getLogger(__name__)
 
 # Sublcass OpenStackConfig in order to munge config values
 # before auth plugins are loaded
-class OSC_Config(OpenStackConfig):
+class OSC_Config(config.OpenStackConfig):
 
     def _auth_select_default_plugin(self, config):
         """Select a default plugin based on supplied arguments
@@ -31,7 +33,7 @@ class OSC_Config(OpenStackConfig):
         Migrated from auth.select_auth_plugin()
         """
 
-        identity_version = config.get('identity_api_version', '')
+        identity_version = str(config.get('identity_api_version', ''))
 
         if config.get('username', None) and not config.get('auth_type', None):
             if identity_version == '3':
@@ -80,8 +82,8 @@ class OSC_Config(OpenStackConfig):
         # NOTE(hieulq): If USER_DOMAIN_NAME, USER_DOMAIN_ID, PROJECT_DOMAIN_ID
         # or PROJECT_DOMAIN_NAME is present and API_VERSION is 2.0, then
         # ignore all domain related configs.
-        if (config.get('identity_api_version', '').startswith('2') and
-                config.get('auth_type', None).endswith('password')):
+        if (str(config.get('identity_api_version', '')).startswith('2') and
+                config.get('auth_type').endswith('password')):
             domain_props = [
                 'project_domain_id',
                 'project_domain_name',
@@ -90,8 +92,15 @@ class OSC_Config(OpenStackConfig):
             ]
             for prop in domain_props:
                 if config['auth'].pop(prop, None) is not None:
-                    LOG.warning("Ignoring domain related config " +
-                                prop + " because identity API version is 2.0")
+                    if config.get('cloud'):
+                        LOG.warning(
+                            "Ignoring domain related config %s for %s"
+                            "because identity API version is 2.0" % (
+                                prop, config['cloud']))
+                    else:
+                        LOG.warning(
+                            "Ignoring domain related config %s because"
+                            " identity API version is 2.0" % prop)
         return config
 
     def _auth_default_domain(self, config):
@@ -100,7 +109,7 @@ class OSC_Config(OpenStackConfig):
         Migrated from clientmanager.setup_auth()
         """
 
-        identity_version = config.get('identity_api_version', '')
+        identity_version = str(config.get('identity_api_version', ''))
         auth_type = config.get('auth_type', None)
 
         # TODO(mordred): This is a usability improvement that's broadly useful
@@ -114,6 +123,7 @@ class OSC_Config(OpenStackConfig):
             # present, then do not change the behaviour. Otherwise, set the
             # PROJECT_DOMAIN_ID to 'OS_DEFAULT_DOMAIN' for better usability.
             if (
+                    auth_type in ("password", "v3password", "v3totp") and
                     not config['auth'].get('project_domain_id') and
                     not config['auth'].get('project_domain_name')
             ):
@@ -145,66 +155,43 @@ class OSC_Config(OpenStackConfig):
         config = self._auth_v2_ignore_v3(config)
         config = self._auth_default_domain(config)
 
-        LOG.debug("auth_config_hook(): %s" % config)
-        return config
-
-    # TODO(dtroyer): un-hackify all of the below when o-c-c 1.19.x is
-    # the minimum in global-requirements
-
-    def get_one_cloud(self, cloud=None, validate=True,
-                      argparse=None, **kwargs):
-
-        # First, save this off for later
-        self._save_argparse = self._fix_args(kwargs, argparse=argparse)
-
-        return super(OSC_Config, self).get_one_cloud(
-            cloud=cloud,
-            validate=validate,
-            argparse=argparse,
-            **kwargs
-        )
-
-    def _validate_auth_ksc(self, config, cloud, fixed_argparse=None):
-        """Old compatibility hack for OSC, no longer needed/wanted"""
+        if LOG.isEnabledFor(logging.DEBUG):
+            LOG.debug("auth_config_hook(): %s",
+                      strutils.mask_password(str(config)))
         return config
 
     def _validate_auth(self, config, loader, fixed_argparse=None):
         """Validate auth plugin arguments"""
         # May throw a keystoneauth1.exceptions.NoMatchingPlugin
 
-        if fixed_argparse is None:
-            # This is o-c-c 1.18.x or older, fix up the original options
-            fixed_argparse = self._fix_args(None, argparse=self._save_argparse)
-
         plugin_options = loader.get_options()
 
+        msgs = []
+        prompt_options = []
         for p_opt in plugin_options:
-            # if it's in argparse, it was passed on the command line and wins
-            # if it's in config.auth, win, kill it from config dict
-            # if it's in config and not in config.auth, move it
+            # if it's in config, win, move it and kill it from config dict
+            # if it's in config.auth but not in config we're good
             # deprecated loses to current
             # provided beats default, deprecated or not
-            winning_value = self._find_winning_auth_value(
-                p_opt, fixed_argparse)
-            if winning_value:
-                found_in_argparse = True
-            else:
-                found_in_argparse = False
+            winning_value = self._find_winning_auth_value(p_opt, config)
+            if not winning_value:
                 winning_value = self._find_winning_auth_value(
                     p_opt, config['auth'])
-                if not winning_value:
-                    winning_value = self._find_winning_auth_value(
-                        p_opt, config)
+
+            # if the plugin tells us that this value is required
+            # then error if it's doesn't exist now
+            if not winning_value and p_opt.required:
+                msgs.append(
+                    'Missing value {auth_key}'
+                    ' required for auth plugin {plugin}'.format(
+                        auth_key=p_opt.name, plugin=config.get('auth_type'),
+                    )
+                )
 
             # Clean up after ourselves
             for opt in [p_opt.name] + [o.name for o in p_opt.deprecated]:
                 opt = opt.replace('-', '_')
-                # don't do this if the value came from argparse, because we
-                # don't (yet) know if the value in not-auth came from argparse
-                # overlay or from someone passing in a dict to kwargs
-                # TODO(mordred) Fix that data path too
-                if not found_in_argparse:
-                    config.pop(opt, None)
+                config.pop(opt, None)
                 config['auth'].pop(opt, None)
 
             if winning_value:
@@ -216,4 +203,28 @@ class OSC_Config(OpenStackConfig):
                 else:
                     config['auth'][p_opt.dest] = winning_value
 
+            # See if this needs a prompting
+            if (
+                    'prompt' in vars(p_opt) and
+                    p_opt.prompt is not None and
+                    p_opt.dest not in config['auth'] and
+                    self._pw_callback is not None
+            ):
+                # Defer these until we know all required opts are present
+                prompt_options.append(p_opt)
+
+        if msgs:
+            raise sdk_exceptions.OpenStackConfigException('\n'.join(msgs))
+        else:
+            for p_opt in prompt_options:
+                config['auth'][p_opt.dest] = self._pw_callback(p_opt.prompt)
+
         return config
+
+    def load_auth_plugin(self, config):
+        """Get auth plugin and validate args"""
+
+        loader = self._get_auth_loader(config)
+        config = self._validate_auth(config, loader)
+        auth_plugin = loader.load_from_options(**config['auth'])
+        return auth_plugin
